/*
 * WScout - Lightweight PCAP visualizer.
 * Copyright (C) 2007, 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */

#ifndef WSCOUT_GUI_IEEE802_11_HEADER_HH_
# define WSCOUT_GUI_IEEE802_11_HEADER_HH_

# include <QtGui/QLabel>

#include <wipal/wifi/frame/dissector/dissector.hh>
#include <wipal/wifi/frame/dissector/default_hooks.hh>

# include "wscout_gui_header_base.hh"

namespace wscout
{

  namespace gui
  {

    /**
     * IEEE 802.11 Widget.
     *
     * @see AbstractHeader, HeaderBase
     */
    struct IEEE80211Header:
      public HeaderBase<IEEE80211Header, protocol::IEEE802_11>
    {
      enum
      {
	before_addresses_spacing	= 16,
	default_spacing			= 4,
	max_duration_len		= 16,
	max_desc_len			= 32
      };

      /// Default constructor.
      IEEE80211Header(QWidget* parent = 0);

      /// Abstract method definition from AbstractHeader.
      virtual next_header	setup(const void*		header,
				      size_t			caplen,
				      const Trace&		trace,
				      const frame_descriptor&	desc);

    protected:
      /// Set this header's colors.
      void			setColor(const QColor fore, const QColor back);

    private:

      struct ieee80211_hooks: public wifi::frame::dissector_default_hooks
      {
	ieee80211_hooks(IEEE80211Header& h);

	void	frame_hook(const void*, size_t);
	void	addr_hook(const void*, size_t, unsigned, const wifi::addr&);
	void	seq_ctl_hook(const void*, size_t, unsigned, unsigned);
	void	end_of_frame_headers_hook(const void*, size_t, const void*);
	void	management_hook(const wifi::frame::mgt::header*, size_t);
	void	control_hook(const void*, size_t);
	void	data_hook(const void*, size_t);
	void	invalid_type_hook(const void*, size_t);
	void	truncated_frame_hook(const void*, size_t,
				     wifi::frame::dissector_status::status);

      protected:

	/// IEEE80211Header colors and frame decription.
	struct attributes
	{
	  std::string	desc;	///< Frame description.
	  QColor	fore;	///< Foreground color.
	  QColor	back;	///< Background color.
	  protocol::id	next;	///< Next protocol.
	};

	void	setDesc(const attributes& attr);
	void	setControl(const void*);

	static const QColor	black;
	static const QColor	lightblue;
	static const QColor	lightgreen;
	static const QColor	red;
	static const QColor	white;
	static const QColor	yellow;

      private:
	IEEE80211Header&	h_;
      };

      next_header	next;

      QLabel		desc;
      QLabel		flags;
      QLabel		duration;
      QLabel		seq_ctl;
      QLabel		addr[4];
      QFrame		addr_sep[3];
    };

  } // End of namespace gui.

} // End of namespace wscout.

#include "wscout_gui_ieee802_11_header.hxx"

#endif // ! WSCOUT_GUI_IEEE802_11_HEADER_HH_
