/*
 * WScout - Lightweight PCAP visualizer.
 * Copyright (C) 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */

#ifndef WSCOUT_GUI_PHY_HEADER_HXX_
# define WSCOUT_GUI_PHY_HEADER_HXX_

# include <boost/lexical_cast.hpp>
# include <boost/format.hpp>

# include "wscout_gui_prism_header.hh"

namespace wscout
{

  namespace gui
  {

    template <class E, int P>
    PHYHeader<E, P>::PHYHeader(const QString&	proto,
			       QWidget*		parent):
      HeaderBase<E, P> (parent),
      proto_str_ (proto)
    {
      this->setUpdatesEnabled(false);

      this->setForegroundRole(QPalette::WindowText);
      this->setBackgroundRole(QPalette::Window);
      this->setAutoFillBackground(true);

      {
	QFont bold;
	bold.setBold(true);

	proto_.setFont(bold);
      }
      time_.setReadOnly(true);
      time_.setFont(QFont ("Fixed"));
      time_.setFixedHeight(time_.fontMetrics().height());
      time_.setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
      time_.setFrame(false);
      {
	QBoxLayout* const	l = this->layout();

	l->addWidget(&proto_);
	l->addSpacing(default_spacing);

	l->addWidget(&time_);
	l->addSpacing(default_spacing);

	l->addWidget(&rate_);
	l->addSpacing(default_spacing);

	l->addWidget(&channel_);
	l->addStretch();

	l->addWidget(&signal_);
      }
      this->setUpdatesEnabled(true);
    }

    template <class E, int P>
    void
    PHYHeader<E, P>::setTruncated(bool trunc)
    {
      static const QColor	trunc_fore (Qt::white),
				trunc_back (Qt::red),
				full_fore ("darkslategrey"),
				full_back (Qt::white);
      QPalette	p;

      if (trunc)
	{
	  p.setColor(QPalette::WindowText, trunc_fore);
	  p.setColor(QPalette::Text, trunc_fore);
	  p.setColor(QPalette::Window, trunc_back);
	  proto_.setText(proto_str_ + " (truncated)");
	}
      else
	{
	  p.setColor(QPalette::WindowText, full_fore);
	  p.setColor(QPalette::Text, full_fore);
	  p.setColor(QPalette::Window, full_back);
	  proto_.setText(proto_str_);
	}

      this->setPalette(p);
    }

    template <class E, int P>
    void
    PHYHeader<E, P>::setTime(uint64_t hosttime, uint64_t mactime)
    {
      const std::string s =
	boost::lexical_cast<std::string> (boost::format ("%010u:%010u")
					  % hosttime
					  % mactime);

      time_.setText(s.c_str());
    }

    template <class E, int P>
    void
    PHYHeader<E, P>::setRate(float rate)
    {
      rate_.setText(this->tr("%1 Mb/s").arg(rate));
    }

    template <class E, int P>
    void
    PHYHeader<E, P>::setChannel(unsigned channel)
    {
      channel_.setText(this->tr("channel %1").arg(channel));
    }
    template <class E, int P>
    void
    PHYHeader<E, P>::setSignal(int rssi, int signal, int noise)
    {
      signal_.setText(this->tr("rssi/signal/noise: %1 / %2 / %3")
		      .arg(rssi >= 0	? QString::number(rssi)	  : "NA")
		      .arg(signal >= 0	? QString::number(signal) : "NA")
		      .arg(noise >= 0	? QString::number(noise)  : "NA"));
    }

  } // End of namespace gui.

} // End of namespace wscout.

#endif // ! WSCOUT_GUI_PRISM_HEADER_HXX_
