/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef PCAP_SIMPLE_TYPES_HH_
# define PCAP_SIMPLE_TYPES_HH_

extern "C"
{
# include <inttypes.h>
# include <time.h>
}

namespace pcapxx
{

  namespace internals
  {
    enum
      {
	/// Current PCAP major version.
	version_major	= 2,

	/// Current PCAP minor version.
	version_minor	= 4
      };

    /// PCAP file format internal header.
    struct file_header
    {
      uint32_t	magic;		///< File magic.
      uint16_t	version_major;	///< PCAP format version, major.
      uint16_t	version_minor;	///< PCAP format version, minor.
      int32_t	thiszone;	///< Time offset.
      int32_t	sigfigs;	///< Timestamp accuracy.
      int32_t	snaplen;	///< Max. number of captured bytes.
      int32_t	linktype;	///< PCAP link type.
    };

    /// PCAP file format representation of a packet header.
    struct file_frame_header
    {
      struct timeval	ts;	///< Timestamp.
      uint32_t		caplen;	///< Number of captured bytes.
      uint32_t		len;	///< %Packet len.
    };

  } // End of namespace pcapxx::internals.

  /**
   * PCAP frame/packet header.
   *
   * A PCAP header holds information regarding a frame/packet that are
   * not directly embedded inside the frame/packet's bytes.
   */
  struct pkthdr
  {
    struct timeval	ts;	///< Time of arrival.
    size_t		caplen;	///< Number of captured bytes.
    size_t		len;	///< Length of the frame/packet.
  };

} // End of namespace pcapxx.

#endif // ! PCAP_SIMPLE_TYPES_HH_

