/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef TOOL_FILTER_HXX_
# define TOOL_FILTER_HXX_

# include "filter.hh"

namespace tool
{

  namespace internals
  {

    template <class I, class P, class B1, class B2>
    filter_iterator<I, P, B1, B2>::filter_iterator(const iterable_type&	i,
						   bool			end):
      current_ (end ? i.last_ : i.first_),
      iterable_ (&i)
    {
      if (not end)
	skip_non_matching_items();
    }

    template <class I, class P, class B1, class B2>
    bool
    filter_iterator<I, P, B1, B2>::equal(const filter_iterator& rhs) const
    {
      return current_ == rhs.current_;
    }

    template <class I, class P, class B1, class B2>
    void
    filter_iterator<I, P, B1, B2>::increment()
    {
      ++current_;
      skip_non_matching_items();
    }

    template <class I, class P, class B1, class B2>
    const typename filter_iterator<I, P, B1, B2>::value_type&
    filter_iterator<I, P, B1, B2>::get() const
    {
      return current_.get();
    }

    template <class I, class P, class B1, class B2>
    typename filter_iterator<I, P, B1, B2>::value_type&
    filter_iterator<I, P, B1, B2>::get()
    {
      return current_.get();
    }

    template <class I, class P, class B1, class B2>
    const typename filter_iterator<I, P, B1, B2>::value_type*
    filter_iterator<I, P, B1, B2>::get_ptr() const
    {
      return current_.get_ptr();
    }

    template <class I, class P, class B1, class B2>
    typename filter_iterator<I, P, B1, B2>::value_type*
    filter_iterator<I, P, B1, B2>::get_ptr()
    {
      return current_.get_ptr();
    }

    template <class I, class P, class B1, class B2>
    void
    filter_iterator<I, P, B1, B2>::skip_non_matching_items()
    {
      while (current_ != iterable_->last_ and
	     not static_cast<const P&> (*iterable_)(*current_))
	++current_;
    }

  } // End of namespace internals.

  template <class I, class P, class B>
  filter<I, P, B>::filter(const I& first,
			  const I& last,
			  const P& p):
    P (p),
    first_ (first),
    last_ (last)
  {
  }

} // End of namespace tool.

#endif // ! TOOL_FILTER_HXX_
