/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef TOOL_HASH_HH_
# define TOOL_HASH_HH_

# ifdef WP_DONT_USE_HASH

#  include <set>
#  include <map>

#  define WP_HASH_NAMESPACE_BEGIN	namespace tool {
#  define WP_HASH_NAMESPACE_END	}

namespace tool
{
  template <class T>
  struct hash
  {
  };
}

# else // ! WP_DONT_USE_HASH

#  ifdef __GNUC__		// GCC

#   if __GNUC__ < 3		// GCC < 3.0
#    include <hash_set.h>
#    include <hash_map.h>

#    define WP_HASH_NAMESPACE_BEGIN
#    define WP_HASH_NAMESPACE_END

namespace tool
{
  using ::hash_map;
  using ::hash_set;
}

				// 3.0 <= GCC < 4.3
#   elif __GNUC__ < 4 || (__GNUC__ == 4 && __GNUC_MINOR__ < 3)
#    include <ext/hash_set>
#    include <ext/hash_map>

				// GCC 3.0
#    if __GNUC__ == 3 && __GNUC_MINOR__ == 0

#     define WP_HASH_NAMESPACE_BEGIN	namespace std {
#     define WP_HASH_NAMESPACE_END		}

namespace tool
{
  using ::std::hash_map;
  using ::std::hash_set;
}

#    else			// 3.1 <= GCC < 4.3

#     define WP_HASH_NAMESPACE_BEGIN	namespace __gnu_cxx {
#     define WP_HASH_NAMESPACE_END		}

namespace tool
{
  using ::__gnu_cxx::hash_map;
  using ::__gnu_cxx::hash_set;
}

#    endif // __GNUC__ == 3 && __GNUC_MINOR__ == 0

#   else			// GCC >= 4.3
#    include <tr1/unordered_set>
#    include <tr1/unordered_map>

#     define WP_HASH_NAMESPACE_BEGIN	namespace std { namespace tr1 {
#     define WP_HASH_NAMESPACE_END		} }

namespace tool
{
  template <class T>
  struct hash_set: public std::tr1::unordered_set<T>
  {
    hash_set()
    {
    }

    hash_set(size_t n): std::tr1::unordered_set<T>(n)
    {
    }
  };

  template <class K, class T>
  struct hash_map: public std::tr1::unordered_map<K, T>
  {
  };
}

#   endif // ! __GNUC__ < 4 || (__GNUC__ == 4 && __GNUC_MINOR__ < 3)

#  else // ! __GNUC__		// ...  There are other compilers, right?

#   include <hash_set>
#   include <hash_map>

#   define WP_HASH_NAMESPACE_BEGIN	namespace std {
#   define WP_HASH_NAMESPACE_END	}

namespace tool
{
  using ::std::hash_map;
  using ::std::hash_set;
}

#  endif // __GNUC__
# endif // WP_DONT_USE_HASH

#endif // ! TOOL_HASH_HH_
