/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_FILTER_MERGE_HH_
# define WIFI_FRAME_FILTER_MERGE_HH_

# include <wipal/phy/prism_header.hh>
# include <wipal/wifi/frame/frame.hh>
# include <wipal/wifi/addr_mapping.hh>
# include <wipal/pcap/frame_descriptor.hh>
# include <wipal/tool/endianness.hh>

namespace wifi
{
  namespace frame
  {
    namespace filter
    {

      namespace internals
      {

	template <class HT, class FD1>
	struct equal
	{
	  equal(const pcapxx::frame_descriptor<FD1>&	lhs,
		const tool::endian::endianness&		phy_end);

	  template <class FD2>
	  bool
	  operator == (const pcapxx::frame_descriptor<FD2>& rhs) const;

	private:
	  const pcapxx::frame_descriptor<FD1>&	lhs_;
	  const tool::endian::endianness&	phy_end_;
	};

	template <class HT, class FD1>
	equal<HT, FD1>
	make_equal(const pcapxx::frame_descriptor<FD1>&	lhs,
		   const tool::endian::endianness&	phy_end);

	template <class HT, class FD1, class FD2>
	bool
	operator == (const pcapxx::frame_descriptor<FD2>&	lhs,
		     const equal<HT, FD1>&			rhs);

      } // End of namespace internals.

      /**
       * Build an merge filter from iterators and pass it as argument
       * to a functor.
       *
       * If @c filter_prism is true, filter elements with @c
       * non_noisy_prism. @c *first and @c *last should be convertible
       * to @c pcapxx::frame_descriptor.
       *
       * @c phy_end indicates the PHY headers' endianness.
       */
      template <class UniqueId,
		class HeaderType,
		template <class, class, class> class Intersector,
		template <class, class, class, class> class Merger,
		class I1, class I2, class F, class BL>
      void
      provide_merge(const I1& first1, const I1&	last1,
		    const I2& first2, const I2&	last2,
		    addr_mapping&		mapping,
		    F&				func,
		    bool			filter_prism,
		    tool::endian::endianness	phy_end,
		    const BL&			blist);

    } // End of namespace wifi::frame::filter.

  } // End of namespace wifi::frame.

} // End of namespace wifi.

# include "merge.hxx"

#endif // ! WIFI_FRAME_FILTER_MERGE_HH_
