/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_STATS_BSS_STATS_HH_
# define WIFI_FRAME_STATS_BSS_STATS_HH_

# include <map>

# include <wipal/wifi/frame/stats/abstract_stats.hh>
# include <wipal/wifi/addr.hh>

namespace wifi
{

  namespace frame
  {

    namespace stats
    {

      struct bss_stats: public abstract_stats
      {
	bss_stats();

	void		account_beacon(const addr& bss, const addr& ap);
	std::ostream&	print(std::ostream& o) const;

      private:
	struct single_bss_stats
	{
	  typedef std::map<addr, unsigned>	ap_stats;

	  single_bss_stats();

	  bool		operator < (const single_bss_stats& rhs) const;

	  unsigned	total_;
	  ap_stats	ap_stats_;
	};

	typedef std::map<wifi::addr, single_bss_stats>	internal_stats;

	unsigned	beacon_count_;
	internal_stats	bss_stats_;
      };

    } // End of namespace wifi::frame::stats.

  } // End of namespace wifi::frame.

} // End of namespace wifi.

# include "bss_stats.hxx"

#endif // ! WIFI_FRAME_STATS_BSS_STATS_HH_
