/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_HASH_HH_
# define WIFI_FRAME_HASH_HH_

# include <openssl/md5.h>
# include <boost/array.hpp>

# include <wipal/wifi/frame/unique_id/unique_id.hh>

namespace wifi
{

  namespace frame
  {

    namespace internals
    {

      struct hash_value: public boost::array<unsigned char, MD5_DIGEST_LENGTH>
      {
	hash_value(const void* data, size_t len);
      };

    } // End of namespace internals.

    typedef unique_id<internals::hash_value>	hsh_id;

    template <>
    struct unique_id_factory<hsh_id>
    {
      template <class HeaderType, class D>
      boost::optional<hsh_id>
      build(const pcapxx::frame_descriptor<D>&	desc,
	    addr_mapping&			mapping);
    };

  } // End of namespace wifi::frame.

} // End of namespace wifi.

WP_HASH_NAMESPACE_BEGIN

  template <>
  struct hash< wifi::frame::internals::hash_value >
  {
    size_t
    operator () (const wifi::frame::internals::hash_value&) const;
  };

WP_HASH_NAMESPACE_END

# include "hash.hxx"

#endif // ! WIFI_FRAME_HASH_HH_
