/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */

#include <iostream>
#include <stdexcept>
#include <boost/bind.hpp>
#include <boost/foreach.hpp>

#include "config.hh"

#include <wipal/wifi/frame/stats/stats.hh>
#include <wipal/tool/options.hh>
#include <wipal/pcap/list_of_traces.hh>
#include <wipal/wifi/frame/filter/microseconds_stamper.hh>
#include <wipal/wifi/frame/filter/non_noisy_prism.hh>

int
main(int argc, char *const argv[])
{
  try
    {
      typedef pcapxx::list_of_traces<>		list_of_traces;
      typedef wifi::frame::stats::stats		stats;

      const tool::opt::options	options =
	tool::opt::parse_args(argc, argv,
			      WP_OPT_ENDIANNESS
			      WP_OPT_PRISM_NOISE,
			      tool::opt::options::arbitrary_number_of_inputs,
			      0);

      stats st;

      BOOST_FOREACH(list_of_traces t, options.input)
	{
	  st.restart();
	  if (options.prism_noise_means_error)
	    t.expect(pcapxx::descriptor<>::PRISM_HEADER);

	  switch (const pcapxx::descriptor<>::link_type lt =
		    t.get(0).linktype())
	    {

#define WP_case(R, Unused, LT)						\
	    case pcapxx::descriptor<>::BOOST_PP_TUPLE_ELEM(2, 0, LT):	\
	      {								\
		using wifi::frame::filter::microseconds_stamper;	\
		using boost::bind;					\
									\
		typedef BOOST_PP_TUPLE_ELEM(2, 1, LT)	phy_header;	\
		typedef list_of_traces::const_iterator	iterator;	\
		typedef microseconds_stamper<iterator, phy_header>	\
							mss_stamper;	\
									\
		mss_stamper s (t.begin(), t.end(), options.endianness); \
									\
		if (options.prism_noise_means_error)			\
		  {							\
		    using wifi::frame::filter::non_noisy_prism;		\
									\
		    typedef mss_stamper::const_iterator		it;	\
		    typedef non_noisy_prism<it>			nnp;	\
		    typedef nnp::const_iterator::value_type	frame;	\
									\
		    nnp	n (s.begin(), s.end());				\
									\
		    n.for_each(bind(&stats::account<phy_header, frame>, \
				    boost::ref(st), _1));		\
		  }							\
		else							\
		  {							\
		    typedef						\
		      mss_stamper::const_iterator::value_type		\
		      frame;						\
									\
		    s.for_each(bind(&stats::account<phy_header, frame>, \
				    boost::ref(st), _1));		\
		  }							\
	      }								\
	    break;

	      BOOST_PP_SEQ_FOR_EACH(WP_case, ~, WP_LINKTYPES)

	    default:
	      tool::opt::unsupported_linktype(lt);
	      break;
	    }
	}

      std::cout << st;
    }
  catch (std::exception& e)
    {
      std::cerr << "ERROR: " << e.what() << '.' << std::endl;
      return EXIT_FAILURE;
    }
}
