#! /bin/sh

exit_code=0

check()
{
    expected=$1
    got=$2

    if [ -f "$expected" ]; then
	if cmp -s "$got" "$expected"; then
	    return 0
	else
	    return 1
	fi
    fi

    return 0
}

fail()
{
    echo "$basename: $1 differs from expected."
    diff -u "$2" "$3"

    last_test=1
    exit_code=1
}

wrap_check_fail()
{
    if ! check "$2" "$3"; then
	fail "$1" "$2" "$3"
    fi
}

for d in "$top_builddir"/src/*/ ; do
    PATH="$d:$PATH"
done

notice()
{
cat<<EOF >&3
NOTICE: check.sh / $basename$@
EOF
}

pass_checkpoint()
{
    echo "PASS: check.sh / $basename / $@" >&3
}

fail_checkpoint()
{
    echo "FAIL: check.sh / $basename / $@" >&3
}

for cmd in "$srcdir"/*.cmd ; do
    prefix=${cmd%.cmd}
    basename=`basename "$prefix"`

    in=$prefix.in

    expected_out=$prefix.out
    expected_err=$prefix.err
    expected_status=$prefix.status

    out=$basename-$$.out
    err=$basename-$$.err
    status=$basename-$$.status

    substed_cmd=`sed "s,@S@,$srcdir/samples/,g" "$cmd"`
    if [ -f "$in" ]; then
	eval 3>&1 "$substed_cmd" < "$in" > "$out" 2> "$err"
	echo "$?" > "$status"
    else
	eval 3>&1 "$substed_cmd" > "$out" 2> "$err"
	echo "$?" > "$status"
    fi

    last_test=0

    wrap_check_fail "Standard output"	"$expected_out"		"$out"
    wrap_check_fail "Standard error "	"$expected_err"		"$err"
    wrap_check_fail "Exit status"	"$expected_status"	"$status"

    test -n "$KEEP_OUTPUTS" || rm "$out" "$err" "$status"
    case $last_test in
	0)
	    echo "PASS: check.sh / $basename"
	    ;;

	1)
	    echo "FAIL: check.sh / $basename"
	    ;;
	*)
	    echo "$0: Internal error!" >&2
	    exit 2
	    ;;
    esac
done

exit $exit_code